@@ -154,14 +154,15 @@ public class ClientImpl
         notifyLifecycleManager();
     }
     
-    public void destroy() {
-        
-        // TODO: also inform the conduit so it can shutdown any response listeners
-        
+    public void destroy() {        
         ClientLifeCycleManager mgr = bus.getExtension(ClientLifeCycleManager.class);
         if (null != mgr) {
             mgr.clientDestroyed(this);
         }
+
+        if (conduitSelector != null) {
+            getConduit().close();
+        }
     }
     
     private void notifyLifecycleManager() {
@@ -503,7 +504,12 @@ public class ClientImpl
                 waitResponse(exchange);
             }
         }
-        getConduitSelector().complete(exchange);
+        
+        // leave the input stream open for the caller
+        Boolean keepConduitAlive = (Boolean)exchange.get(Client.KEEP_CONDUIT_ALIVE);
+        if (keepConduitAlive == null || !keepConduitAlive) {
+            getConduitSelector().complete(exchange);
+        }
 
         // Grab the response objects if there are any
         List resList = null;
@@ -536,7 +542,9 @@ public class ClientImpl
             return exchange.getInFaultMessage().getContent(Exception.class);
         } else if (exchange.getOutFaultMessage() != null) {
             return exchange.getOutFaultMessage().getContent(Exception.class);
-        } 
+        } else if (exchange.getInMessage() != null) {
+            return exchange.getInMessage().getContent(Exception.class);
+        }
         return null;
     }
 
